import enum


# Source: https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes
class CountryEnum(str, enum.Enum):
    AF = "Afghanistan"
    AX = "land Islands"
    AL = "Albania"
    DZ = "Algeria"
    AS = "American Samoa"
    AD = "AndorrA"
    AO = "Angola"
    AI = "Anguilla"
    AQ = "Antarctica"
    AG = "Antigua and Barbuda"
    AR = "Argentina"
    AM = "Armenia"
    AW = "Aruba"
    AU = "Australia"
    AT = "Austria"
    AZ = "Azerbaijan"
    BS = "Bahamas"
    BH = "Bahrain"
    BD = "Bangladesh"
    BB = "Barbados"
    BY = "Belarus"
    BE = "Belgium"
    BZ = "Belize"
    BJ = "Benin"
    BM = "Bermuda"
    BT = "Bhutan"
    BO = "Bolivia"
    BA = "Bosnia and Herzegovina"
    BW = "Botswana"
    BV = "Bouvet Island"
    BR = "Brazil"
    IO = "British Indian Ocean Territory"
    BN = "Brunei Darussalam"
    BG = "Bulgaria"
    BF = "Burkina Faso"
    BI = "Burundi"
    KH = "Cambodia"
    CM = "Cameroon"
    CA = "Canada"
    CV = "Cape Verde"
    KY = "Cayman Islands"
    CF = "Central African Republic"
    TD = "Chad"
    CL = "Chile"
    CN = "China"
    CX = "Christmas Island"
    CC = "Cocos (Keeling) Islands"
    CO = "Colombia"
    KM = "Comoros"
    CG = "Congo"
    CD = "Congo, The Democratic Republic of the"
    CK = "Cook Islands"
    CR = "Costa Rica"
    CI = 'Cote D"Ivoire'
    HR = "Croatia"
    CU = "Cuba"
    CY = "Cyprus"
    CZ = "Czech Republic"
    DK = "Denmark"
    DJ = "Djibouti"
    DM = "Dominica"
    DO = "Dominican Republic"
    EC = "Ecuador"
    EG = "Egypt"
    SV = "El Salvador"
    GQ = "Equatorial Guinea"
    ER = "Eritrea"
    EE = "Estonia"
    ET = "Ethiopia"
    FK = "Falkland Islands (Malvinas)"
    FO = "Faroe Islands"
    FJ = "Fiji"
    FI = "Finland"
    FR = "France"
    GF = "French Guiana"
    PF = "French Polynesia"
    TF = "French Southern Territories"
    GA = "Gabon"
    GM = "Gambia"
    GE = "Georgia"
    DE = "Germany"
    GH = "Ghana"
    GI = "Gibraltar"
    GR = "Greece"
    GL = "Greenland"
    GD = "Grenada"
    GP = "Guadeloupe"
    GU = "Guam"
    GT = "Guatemala"
    GG = "Guernsey"
    GN = "Guinea"
    GW = "Guinea-Bissau"
    GY = "Guyana"
    HT = "Haiti"
    HM = "Heard Island and Mcdonald Islands"
    VA = "Holy See (Vatican City State)"
    HN = "Honduras"
    HK = "Hong Kong"
    HU = "Hungary"
    IS = "Iceland"
    IN = "India"
    ID = "Indonesia"
    IR = "Iran, Islamic Republic Of"
    IQ = "Iraq"
    IE = "Ireland"
    IM = "Isle of Man"
    IL = "Israel"
    IT = "Italy"
    JM = "Jamaica"
    JP = "Japan"
    JE = "Jersey"
    JO = "Jordan"
    KZ = "Kazakhstan"
    KE = "Kenya"
    KI = "Kiribati"
    KP = 'Korea, Democratic People"S Republic of'
    KR = "Korea, Republic of"
    KW = "Kuwait"
    KG = "Kyrgyzstan"
    LA = 'Lao People"S Democratic Republic'
    LV = "Latvia"
    LB = "Lebanon"
    LS = "Lesotho"
    LR = "Liberia"
    LY = "Libyan Arab Jamahiriya"
    LI = "Liechtenstein"
    LT = "Lithuania"
    LU = "Luxembourg"
    MO = "Macao"
    MK = "Macedonia, The Former Yugoslav Republic of"
    MG = "Madagascar"
    MW = "Malawi"
    MY = "Malaysia"
    MV = "Maldives"
    ML = "Mali"
    MT = "Malta"
    MH = "Marshall Islands"
    MQ = "Martinique"
    MR = "Mauritania"
    MU = "Mauritius"
    YT = "Mayotte"
    MX = "Mexico"
    FM = "Micronesia, Federated States of"
    MD = "Moldova, Republic of"
    MC = "Monaco"
    MN = "Mongolia"
    ME = "Montenegro"
    MS = "Montserrat"
    MA = "Morocco"
    MZ = "Mozambique"
    MM = "Myanmar"
    NA = "Namibia"
    NR = "Nauru"
    NP = "Nepal"
    NL = "Netherlands"
    AN = "Netherlands Antilles"
    NC = "New Caledonia"
    NZ = "New Zealand"
    NI = "Nicaragua"
    NE = "Niger"
    NG = "Nigeria"
    NU = "Niue"
    NF = "Norfolk Island"
    MP = "Northern Mariana Islands"
    NO = "Norway"
    OM = "Oman"
    PK = "Pakistan"
    PW = "Palau"
    PS = "Palestinian Territory, Occupied"
    PA = "Panama"
    PG = "Papua New Guinea"
    PY = "Paraguay"
    PE = "Peru"
    PH = "Philippines"
    PN = "Pitcairn"
    PL = "Poland"
    PT = "Portugal"
    PR = "Puerto Rico"
    QA = "Qatar"
    RE = "Reunion"
    RO = "Romania"
    RU = "Russian Federation"
    RW = "RWANDA"
    SH = "Saint Helena"
    KN = "Saint Kitts and Nevis"
    LC = "Saint Lucia"
    PM = "Saint Pierre and Miquelon"
    VC = "Saint Vincent and the Grenadines"
    WS = "Samoa"
    SM = "San Marino"
    ST = "Sao Tome and Principe"
    SA = "Saudi Arabia"
    SN = "Senegal"
    RS = "Serbia"
    SC = "Seychelles"
    SL = "Sierra Leone"
    SG = "Singapore"
    SK = "Slovakia"
    SI = "Slovenia"
    SB = "Solomon Islands"
    SO = "Somalia"
    ZA = "South Africa"
    GS = "South Georgia and the South Sandwich Islands"
    ES = "Spain"
    LK = "Sri Lanka"
    SD = "Sudan"
    SR = "Suriname"
    SJ = "Svalbard and Jan Mayen"
    SZ = "Swaziland"
    SE = "Sweden"
    CH = "Switzerland"
    SY = "Syrian Arab Republic"
    TW = "Taiwan, Province of China"
    TJ = "Tajikistan"
    TZ = "Tanzania, United Republic of"
    TH = "Thailand"
    TL = "Timor-Leste"
    TG = "Togo"
    TK = "Tokelau"
    TO = "Tonga"
    TT = "Trinidad and Tobago"
    TN = "Tunisia"
    TR = "Turkey"
    TM = "Turkmenistan"
    TC = "Turks and Caicos Islands"
    TV = "Tuvalu"
    UG = "Uganda"
    UA = "Ukraine"
    AE = "United Arab Emirates"
    GB = "United Kingdom"
    US = "United States"
    UM = "United States Minor Outlying Islands"
    UY = "Uruguay"
    UZ = "Uzbekistan"
    VU = "Vanuatu"
    VE = "Venezuela"
    VN = "Viet Nam"
    VG = "Virgin Islands, British"
    VI = "Virgin Islands, U.S."
    WF = "Wallis and Futuna"
    EH = "Western Sahara"
    YE = "Yemen"
    ZM = "Zambia"
    ZW = "Zimbabwe"


# :: https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes
class LanguageEnum(str, enum.Enum):
    AA = "Afar"
    AB = "Abkhazian"
    AF = "Afrikaans"
    AK = "Akan"
    AM = "Amharic"
    AN = "Aragonese"
    AR = "Arabic"
    AS = "Assamese"
    AV = "Avar"
    AY = "Aymara"
    AZ = "Azerbaijani"
    BA = "Bashkir"
    BE = "Belarusian"
    BG = "Bulgarian"
    BH = "Bihari"
    BI = "Bislama"
    BM = "Bambara"
    BN = "Bengali"
    BO = "Tibetan"
    BR = "Breton"
    BS = "Bosnian"
    CA = "Catalan"
    CE = "Chechen"
    CH = "Chamorro"
    CO = "Corsican"
    CR = "Cree"
    CS = "Czech"
    CU = "Old Church Slavonic / Old Bulgarian"
    CV = "Chuvash"
    CY = "Welsh"
    DA = "Danish"
    DE = "Deutsch"
    DV = "Divehi"
    DZ = "Dzongkha"
    EE = "Ewe"
    EL = "Greek"
    EN = "English"
    EO = "Esperanto"
    ES = "Spanish"
    ET = "Estonian"
    EU = "Basque"
    FA = "Persian"
    FF = "Peul"
    FI = "Finnish"
    FJ = "Fijian"
    FO = "Faroese"
    FR = "French"
    FY = "West Frisian"
    GA = "Irish"
    GD = "Scottish Gaelic"
    GL = "Galician"
    GN = "Guarani"
    GU = "Gujarati"
    GV = "Manx"
    HA = "Hausa"
    HE = "Hebrew"
    HI = "Hindi"
    HO = "Hiri Motu"
    HR = "Croatian"
    HT = "Haitian"
    HU = "Hungarian"
    HY = "Armenian"
    HZ = "Herero"
    IA = "Interlingua"
    ID = "Indonesian"
    IE = "Interlingue"
    IG = "Igbo"
    II = "Sichuan Yi"
    IK = "Inupiak"
    IO = "Ido"
    IS = "Icelandic"
    IT = "Italian"
    IU = "Inuktitut"
    JA = "Japanese"
    JV = "Javanese"
    KA = "Georgian"
    KG = "Kongo"
    KI = "Kikuyu"
    KJ = "Kuanyama"
    KK = "Kazakh"
    KL = "Greenlandic"
    KM = "Cambodian"
    KN = "Kannada"
    KO = "Korean"
    KR = "Kanuri"
    KS = "Kashmiri"
    KU = "Kurdish"
    KV = "Komi"
    KW = "Cornish"
    KY = "Kirghiz"
    LA = "Latin"
    LB = "Luxembourgish"
    LG = "Ganda"
    LI = "Limburgian"
    LN = "Lingala"
    LO = "Laotian"
    LT = "Lithuanian"
    LU = "Luba-Katanga"
    LV = "Latvian"
    MG = "Malagasy"
    MH = "Marshallese"
    MI = "Maori"
    MK = "Macedonian"
    ML = "Malayalam"
    MN = "Mongolian"
    MO = "Moldovan"
    MR = "Marathi"
    MS = "Malay"
    MT = "Maltese"
    MY = "Burmese"
    NA = "Nauruan"
    NB = "Norwegian Bokmål"
    ND = "North Ndebele"
    NE = "Nepali"
    NG = "Ndonga"
    NL = "Dutch"
    NN = "Norwegian Nynorsk"
    NO = "Norwegian"
    NR = "South Ndebele"
    NV = "Navajo"
    NY = "Chichewa"
    OC = "Occitan"
    OJ = "Ojibwa"
    OM = "Oromo"
    OR = "Oriya"
    OS = "Ossetian / Ossetic"
    PA = "Panjabi / Punjabi"
    PI = "Pali"
    PL = "Polish"
    PS = "Pashto"
    PT = "Portuguese"
    QU = "Quechua"
    RM = "Raeto Romance"
    RN = "Kirundi"
    RO = "Romanian"
    RU = "Russian"
    RW = "Rwandi"
    SA = "Sanskrit"
    SC = "Sardinian"
    SD = "Sindhi"
    SE = "Northern Sami"
    SG = "Sango"
    SH = "Serbo-Croatian"
    SI = "Sinhalese"
    SK = "Slovak"
    SL = "Slovenian"
    SM = "Samoan"
    SN = "Shona"
    SO = "Somalia"
    SQ = "Albanian"
    SR = "Serbian"
    SS = "Swati"
    ST = "Southern Sotho"
    SU = "Sundanese"
    SV = "Swedish"
    SW = "Swahili"
    TA = "Tamil"
    TE = "Telugu"
    TG = "Tajik"
    TH = "Thai"
    TI = "Tigrinya"
    TK = "Turkmen"
    TL = "Tagalog / Filipino"
    TN = "Tswana"
    TO = "Tonga"
    TR = "Turkish"
    TS = "Tsonga"
    TT = "Tatar"
    TW = "Twi"
    TY = "Tahitian"
    UG = "Uyghur"
    UK = "Ukrainian"
    UR = "Urdu"
    UZ = "Uzbek"
    VE = "Venda"
    VI = "Vietnamese"
    VO = "Volapük"
    WA = "Walloon"
    WO = "Wolof"
    XH = "Xhosa"
    YI = "Yiddish"
    YO = "Yoruba"
    ZA = "Zhuang"
    ZH = "Chinese"
    ZU = "Zulu"



USER_OBJ = {
    "auth0|testing": {
        "created_at": "2023-08-15T13:25:31.507Z",
        "email": "test@test.org",
        "email_verified": True,
        "identities": [
            {
                "connection": "Username-Password-Authentication",
                "provider": "auth0",
                "user_id": "testing",
                "isSocial": False,
            }
        ],
        "name": "Test Tester",
        "nickname": "testing",
        "picture": "https://avatars.githubusercontent.com/u/15138480?v=4",
        "updated_at": "2024-01-17T12:36:37.300Z",
        "user_id": "auth0|testing",
        "user_metadata": {},
        "last_password_reset": "2024-01-17T11:42:08.761Z",
        "last_ip": "127.0.0.1",
        "last_login": "2024-01-17T11:43:09.620Z",
        "logins_count": 1,
    },
    "auth0|new_user": {
        "created_at": "2023-08-15T13:25:31.507Z",
        "email": "test2@test.org",
        "email_verified": True,
        "identities": [
            {
                "connection": "Username-Password-Authentication",
                "provider": "auth0",
                "user_id": "testing",
                "isSocial": False,
            }
        ],
        "name": "Test Tester 2",
        "nickname": "testing 2",
        "picture": "https://avatars.githubusercontent.com/u/15138481?v=4",
        "updated_at": "2024-01-17T12:36:37.303Z",
        "user_id": "auth0|new_user",
        "user_metadata": {},
        "last_password_reset": "2024-01-17T11:42:08.759Z",
        "last_ip": "127.0.0.1",
        "last_login": "2024-01-17T11:43:09.618Z",
        "logins_count": 1,
    },
}


def get_user_auth0(sub, *args, **kwargs) -> dict:
    return USER_OBJ[sub]


def patch_user_auth0(input_obj: dict, sub, *args, **kwargs) -> dict:
    USER_OBJ[sub].update(input_obj)
    return get_user_auth0(sub)


def get_user_auth0_metadata(sub, *args, **kwargs) -> dict:
    return USER_OBJ[sub]["user_metadata"]


def check_company_auth0(*args, **kwargs) -> bool:
    return True


def auth0_sub_to_profile(sub: str) -> dict:
    return {
        "email": USER_OBJ[sub]["email"],
        "name": USER_OBJ[sub]["name"],
        "picture": USER_OBJ[sub]["picture"],
        "company_ids": USER_OBJ[sub]["user_metadata"]["company_ids"],
    }


def auth0_sub_to_public(sub: str) -> dict:
    return {
        "email": USER_OBJ[sub]["email"],
        "name": USER_OBJ[sub]["name"],
        "picture": USER_OBJ[sub]["picture"],
    }


def patch_user_auth0_metadata(input_obj: dict, sub, *args, **kwargs) -> dict:
    USER_OBJ[sub]["user_metadata"].update(input_obj)
    return get_user_auth0_metadata(sub)


def set_company_id(sub: str, company_id: str):
    if sub not in USER_OBJ:
        USER_OBJ[sub] = {}
    USER_OBJ[sub]["user_metadata"] = {"company_ids": [company_id]}



import os

import boto3
from botocore.exceptions import ClientError

AWS_CLIENT_ID = os.getenv("AWS_CLIENT_ID")
AWS_CLIENT_SECRET = os.getenv("AWS_CLIENT_SECRET")
AWS_SENDER_EMAIL = os.getenv("AWS_SENDER_EMAIL")
AWS_REGION = os.getenv("AWS_REGION", "eu-central-1")


async def send_email_ses(recipient_email, subject, html_body):
    ses_client = boto3.client(
        "ses",
        aws_access_key_id=AWS_CLIENT_ID,
        aws_secret_access_key=AWS_CLIENT_SECRET,
        region_name=AWS_REGION,
    )
    email_message = {
        "Source": AWS_SENDER_EMAIL,
        "Destination": {"ToAddresses": [recipient_email]},
        "Message": {
            "Subject": {"Data": subject, "Charset": "UTF-8"},
            "Body": {"Html": {"Data": html_body, "Charset": "UTF-8"}},
        },
    }

    try:
        response = ses_client.send_email(**email_message)
        return response["MessageId"]
    except ClientError as e:
        return None



from fastapi import HTTPException, status


class UnauthorizedException(HTTPException):
    def __init__(self, detail: str, **kwargs):
        """Returns HTTP 403"""
        super().__init__(status.HTTP_403_FORBIDDEN, detail=detail)


class UnauthenticatedException(HTTPException):
    def __init__(self):
        super().__init__(status_code=status.HTTP_401_UNAUTHORIZED, detail="Requires authentication")



from typing import Optional, Type, Union, get_args, get_origin, get_type_hints

from pydantic import BaseModel, create_model


def optional_fields(cls: Type[BaseModel]) -> Type[BaseModel]:
    fields = {}
    for name, hint in get_type_hints(cls).items():
        if name.startswith("_"):
            continue

        if get_origin(hint) is not Union or type(None) not in get_args(hint):
            hint = Optional[hint]

        fields[name] = (hint, None)

    new_model = create_model(cls.__name__, __base__=cls, **fields)

    return new_model



from typing import Generator

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm.session import Session

from .common import SQLALCHEMY_DATABASE_URL, name, ssl_mode

engine = create_engine(
    SQLALCHEMY_DATABASE_URL + name, pool_pre_ping=True, connect_args={"sslmode": ssl_mode}
)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)


def get_db() -> Generator[Session, None, None]:
    with SessionLocal() as db:
        yield db
